@@ -9,15 +9,17 @@ impl Service for ServiceHandler {
     type Data = ();
     async fn handle(
         &self,
-        req: &mut Request,
+        request: &Request,
         _ctx: &Self::Data,
     ) -> Result<Option<Response>> {
-        let mut response = None;
-        if req.matches("hello") {
-            let params: String = req.deserialize()?;
-            let message = format!("Hello, {}!", params);
-            response = Some((req, Value::String(message)).into());
-        }
+        let response = match request.method() {
+            "hello" => {
+                let params: String = request.deserialize()?;
+                let message = format!("Hello, {}!", params);
+                Some((request, Value::String(message)).into())
+            }
+            _ => None,
+        };
         Ok(response)
     }
 }
@@ -25,10 +27,10 @@ impl Service for ServiceHandler {
 #[tokio::main]
 async fn main() -> Result<()> {
     let service: Box<dyn Service<Data = ()>> = Box::new(ServiceHandler {});
-    let mut request =
+    let request =
         Request::new_reply("hello", Some(Value::String("world".to_string())));
     let server = Server::new(vec![&service]);
-    let response = server.serve(&mut request, &()).await;
+    let response = server.serve(&request, &()).await;
     println!("{:?}", response.as_ref().unwrap().result());
     assert_eq!(
         Some(Value::String("Hello, world!".to_string())),
