@@ -1,4 +1,5 @@
 #![deny(missing_docs)]
+#![allow(clippy::borrowed_box)]
 //! Simple, robust and pragmatic facade for JSON-RPC2 services that is transport agnostic.
 //!
 //! ```
@@ -116,9 +117,9 @@ pub enum Error {
     Boxed(#[from] Box<dyn std::error::Error + Send + Sync>),
 }
 
-impl<'a> Into<(isize, Option<String>)> for &'a Error {
-    fn into(self) -> (isize, Option<String>) {
-        match self {
+impl<'a> From<&'a Error> for (isize, Option<String>) {
+    fn from(error: &'a Error) -> Self {
+        match error {
             Error::MethodNotFound { .. } => (METHOD_NOT_FOUND, None),
             Error::InvalidParams { data, .. } => {
                 (INVALID_PARAMS, Some(data.to_string()))
@@ -132,38 +133,6 @@ impl<'a> Into<(isize, Option<String>)> for &'a Error {
     }
 }
 
-impl<'a> Into<Response> for (&'a Request, Error) {
-    fn into(self) -> Response {
-        let (code, data): (isize, Option<String>) = (&self.1).into();
-        Response {
-            jsonrpc: VERSION.to_string(),
-            id: self.0.id.clone(),
-            result: None,
-            error: Some(RpcError {
-                code,
-                message: self.1.to_string(),
-                data,
-            }),
-        }
-    }
-}
-
-impl Into<Response> for Error {
-    fn into(self) -> Response {
-        let (code, data): (isize, Option<String>) = (&self).into();
-        Response {
-            jsonrpc: VERSION.to_string(),
-            id: Some(Value::Null),
-            result: None,
-            error: Some(RpcError {
-                code,
-                message: self.to_string(),
-                data,
-            }),
-        }
-    }
-}
-
 impl<'a> From<(&'a mut Request, &'a str)> for Error {
     fn from(value: (&'a mut Request, &'a str)) -> Error {
         Error::from((value.0, value.1.to_string()))
@@ -192,6 +161,17 @@ pub struct RpcError {
     pub data: Option<String>,
 }
 
+impl RpcError {
+    /// Create a new JSON-RPC internal error.
+    pub fn new(message: String, data: Option<String>) -> Self {
+        Self {
+            code: INTERNAL_ERROR,
+            message,
+            data,
+        }
+    }
+}
+
 /// Trait for services that maybe handle a request.
 pub trait Service: Send + Sync {
     /// Type of the user data for this service.
@@ -265,23 +245,22 @@ impl<'a, T> Server<'a, T> {
 
 /// Parse a JSON payload from a string slice into a request.
 pub fn from_str(payload: &str) -> Result<Request> {
-    Ok(serde_json::from_str::<Request>(payload).map_err(map_json_error)?)
+    serde_json::from_str::<Request>(payload).map_err(map_json_error)
 }
 
 /// Parse a JSON payload from a [Value](serde_json::Value) into a request.
 pub fn from_value(payload: Value) -> Result<Request> {
-    Ok(serde_json::from_value::<Request>(payload).map_err(map_json_error)?)
+    serde_json::from_value::<Request>(payload).map_err(map_json_error)
 }
 
 /// Parse a JSON payload from a byte slice into a request.
-pub fn from_slice<'a>(payload: &'a [u8]) -> Result<Request> {
-    Ok(serde_json::from_slice::<Request>(payload).map_err(map_json_error)?)
+pub fn from_slice(payload: &[u8]) -> Result<Request> {
+    serde_json::from_slice::<Request>(payload).map_err(map_json_error)
 }
 
 /// Parse a JSON payload from an IO reader into a request.
 pub fn from_reader<R: std::io::Read>(payload: R) -> Result<Request> {
-    Ok(serde_json::from_reader::<R, Request>(payload)
-        .map_err(map_json_error)?)
+    serde_json::from_reader::<R, Request>(payload).map_err(map_json_error)
 }
 
 /// JSON-RPC request.
@@ -360,7 +339,7 @@ impl Request {
     #[deprecated(note = "Use match expression on method() instead")]
     /// Determine if the given name matches the request method.
     pub fn matches(&self, name: &str) -> bool {
-        name == &self.method
+        *name == self.method
     }
 
     /// Deserialize and consume the message parameters into type `T`.
@@ -426,21 +405,64 @@ impl Response {
     }
 }
 
-impl Into<(Option<Value>, Option<RpcError>, Option<Value>)> for Response {
-    fn into(self) -> (Option<Value>, Option<RpcError>, Option<Value>) {
-        (self.id, self.error, self.result)
+impl From<Response> for (Option<Value>, Option<RpcError>, Option<Value>) {
+    fn from(response: Response) -> Self {
+        (response.id, response.error, response.result)
     }
 }
 
-impl Into<Option<Value>> for Response {
-    fn into(self) -> Option<Value> {
-        self.result
+impl From<Response> for Option<Value> {
+    fn from(response: Response) -> Self {
+        response.result
     }
 }
 
-impl Into<Option<RpcError>> for Response {
-    fn into(self) -> Option<RpcError> {
-        self.error
+impl From<Response> for Option<RpcError> {
+    fn from(response: Response) -> Self {
+        response.error
+    }
+}
+
+impl From<Error> for Response {
+    fn from(error: Error) -> Self {
+        let (code, data): (isize, Option<String>) = (&error).into();
+        Response {
+            jsonrpc: VERSION.to_string(),
+            id: Some(Value::Null),
+            result: None,
+            error: Some(RpcError {
+                code,
+                message: error.to_string(),
+                data,
+            }),
+        }
+    }
+}
+
+impl<'a> From<(&'a Request, Error)> for Response {
+    fn from(result: (&'a Request, Error)) -> Self {
+        let (code, data): (isize, Option<String>) = (&result.1).into();
+        Response {
+            jsonrpc: VERSION.to_string(),
+            id: result.0.id.clone(),
+            result: None,
+            error: Some(RpcError {
+                code,
+                message: result.1.to_string(),
+                data,
+            }),
+        }
+    }
+}
+
+impl<'a> From<(&'a Request, RpcError)> for Response {
+    fn from(result: (&'a Request, RpcError)) -> Self {
+        Response {
+            jsonrpc: VERSION.to_string(),
+            id: result.0.id.clone(),
+            result: None,
+            error: Some(result.1),
+        }
     }
 }
 
@@ -521,6 +543,20 @@ mod test {
         }
     }
 
+    struct InternalRpcErrorService;
+    impl Service for InternalRpcErrorService {
+        type Data = ();
+        fn handle(
+            &self,
+            request: &Request,
+            _context: &Self::Data,
+        ) -> Result<Option<Response>> {
+            let err = RpcError::new("Mock RPC error".to_string(), Some("close-connection".to_string()));
+            let res = Some((request, err).into());
+            Ok(res)
+        }
+    }
+
     #[test]
     fn jsonrpc_service_ok() -> Result<()> {
         let service: Box<dyn Service<Data = ()>> =
@@ -649,4 +685,23 @@ mod test {
         );
         Ok(())
     }
+
+    #[test]
+    fn jsonrpc_internal_rpc_error() -> Result<()> {
+    
+        let service: Box<dyn Service<Data = ()>> =
+            Box::new(InternalRpcErrorService {});
+        let request = Request::new_reply("foo", None);
+        let server = Server::new(vec![&service]);
+        let response = server.serve(&request, &());
+        assert_eq!(
+            Some(RpcError {
+                code: -32603,
+                message: "Mock RPC error".to_string(),
+                data: Some("close-connection".to_string())
+            }),
+            response.unwrap().into()
+        );
+        Ok(())
+    }
 }
